@@ -7,17 +7,26 @@ class Config(object):
     CM configuration class
     """
 
-    def __init__(self, config_file: str = None):
+    def __init__(self, config_file = None):
         """
-        Initialize Collective Mind configuration
+        Initialize CM configuration class
+
+        Args:
+            (config_file) (str): If not None, merge this file with the internal configuration
+
+        Returns:
+            (python class) with the following vars:
+
+            * cfg (dict): internal CM configuration
         """
 
         self.cfg = {
                      "name": "cm",
 
                      "env_home": "CM_HOME",
+                     "env_repos": "CM_REPOS",
                      "env_debug": "CM_DEBUG",
-                     "env_core": "CM_CORE",
+                     "env_config": "CM_CONFIG",
 
                      "flag_debug": "cm-debug",
 
@@ -41,13 +50,13 @@ class Config(object):
                        "ls":"search",
                        "list":"search",
                        "find":"search",
-                       "rm":"delete"
+                       "rm":"delete",
+                       "mv":"move",
+                       "ren":"move",
+                       "rename":"move"
                      },
 
-                     "action_substitutions_reverse": {
-                       "search":["list", "ls", "find"],
-                       "delete":["rm"]
-                     },
+                     "new_repo_requirements": "cmind >= 0.7.5\n",
 
                      "cmind_automation":"automation",
 
@@ -71,7 +80,7 @@ class Config(object):
         # Attempt to update config from file if specified explicitly during initialization
         # or specified by the environment variable
         if config_file is None or config_file.strip() == '':
-            config_file = os.environ.get(self.cfg['env_core'])
+            config_file = os.environ.get(self.cfg['env_config'])
 
         if config_file is not None and config_file.strip() != '':
             from cmind import utils
