@@ -17,18 +17,38 @@ cm = None
 ############################################################
 class CM(object):
     """
-    Main Collective Mind class
+    Main CM class
     """
 
     ############################################################
-    def __init__(self, home_path = '', debug = False):
-        """
-        Initialize Collective Mind class
+    def __init__(self, repos_path = '', debug = False):
         """
+        Initialize CM configuration class
 
-#        print ('************************************************')
-#        print ('Initialize CM ...')
-#        print ('************************************************')
+        Args:
+            (repos_path) (str) - path to CM repositories and other information.
+            (debug) (bool) - if True, raise errors in internal functions 
+                             instead of returning a dictionary with "return">0.
+
+        Returns:
+            (python class) with the following vars:
+
+            * cfg (dict): internal CM configuration
+            * debug (bool): debug state
+
+            * repos_path (str): path to CM repositories and other information
+
+            * path_to_cmind (str): path to the used CM toolkit
+            * path_to_cmind_core_module (str): path to CM core module
+            * path_to_cmind_repo (str): path to the "internal" CM repo
+
+            * repos (list): list of initialized CM repository objects
+            
+            * common_automation (Automation class): initialized common automation with actions when a given automation is not found or doesn't exist
+
+            * output (str): value of --out during the first access to CM (to print errors, etc)
+
+        """
 
         # Initialize and update CM configuration
         self.cfg = Config().cfg
@@ -39,18 +59,24 @@ class CM(object):
             self.debug = True
 
         # Explicit path to direcory with CM repositories and other internals
-        self.home_path = home_path
-        if self.home_path == '':
-            s = os.environ.get(self.cfg['env_home'],'').strip()
+        self.repos_path = repos_path
+        if self.repos_path == '':
+            s = os.environ.get(self.cfg['env_repos'],'').strip()
             if s != '':
-                self.home_path = s
+                self.repos_path = s
 
-        if self.home_path == '':
+        if self.repos_path == '':
            from os.path import expanduser
-           self.home_path = os.path.join(expanduser("~"), self.cfg['default_home_dir'])
+           self.repos_path = os.path.join(expanduser("~"), self.cfg['default_home_dir'])
+
+        path_to_cmind = os.environ.get(self.cfg['env_home'],'').strip()
+        if path_to_cmind != '':
+            self.path_to_cmind = path_to_cmind
+            self.path_to_cmind_core_module = os.path.join(self.path_to_cmind, 'core.py')
+        else:
+            self.path_to_cmind_core_module = inspect.getfile(inspect.currentframe())
+            self.path_to_cmind = os.path.dirname(self.path_to_cmind_core_module)
 
-        self.path_to_cmind_kernel = inspect.getfile(inspect.currentframe())
-        self.path_to_cmind = os.path.dirname(self.path_to_cmind_kernel)
         self.path_to_cmind_repo = os.path.join(self.path_to_cmind, self.cfg['cmind_repo'])
 
         # Repositories
@@ -65,14 +91,13 @@ class CM(object):
     ############################################################
     def error(self, r):
         """
-        Process CM error (print or raise and exit)
+        If r['return']>0: print CM error and raise error if in debugging mode
 
-            Args:
-               r (dict)        - output from CM function
-               debug (Boolean) - if True, call raise
+        Args:
+           r (dict): output from CM function with "return" and "error"
                
         Returns:
-            None or raise
+           (dict): r
 
         """
 
@@ -89,15 +114,13 @@ class CM(object):
     ############################################################
     def halt(self, r):
         """
-        Process CM error and halt (useful for scripts)
+        If r['return']>0: print CM error and raise error if in debugging mode or halt with "return" code
 
-            Args:
-               r (dict)        - output from CM function
-               debug (Boolean) - if True, call raise
+        Args:
+           r (dict): output from CM function with "return" and "error"
                
         Returns:
-            None or raise
-
+           (dict): r
         """
 
         # Force console
@@ -108,22 +131,32 @@ class CM(object):
     ############################################################
     def access(self, i, out = None):
         """
-        Access customized Collective Mind object
+        Access CM automation actions in a unified way similar to micro-services.
+
+        i (dict | str | argv): unified CM input
 
         Args:
-            i (dict | str | argv): CM input
-            out (str) - =='con' -> force output to console
+            (action) (str): automation action
+            (automation (CM object): CM automation in format (alias | UID | alias,UID) 
+                                       or (repo alias | repo UID | repo alias,UID):(alias | UID | alias,UID) 
+            (artifact) (CM object): CM artifact
+            (artifacts) (list of CM objects): extra CM artifacts
 
-            (common) (bool) - if True force common automation action
+            (common) (bool): if True, use common automation action from Automation class
 
-        Returns:
-            Dictionary:
-                return (int): return code == 0 if no error 
-                                          >0 if error
+            (help) (bool): if True, print CM automation action API
 
-                (error) (str): error string if return>0
+            (ignore_inheritance) (bool): if True, ignore inheritance when searching for artifacts and automations
+            
+            (out) (str): if 'out', tell automations and CM to output extra information to console
 
-                data from a given action
+        Returns: 
+            (CM return dict):
+
+            * return (int): return code == 0 if no error and >0 if error
+            * (error) (str): error string if return>0
+
+            * Output from a CM automation action
         
         """
 
@@ -180,13 +213,13 @@ class CM(object):
                 print (self.cfg['info_cli'])
 
                 if cm_help or extra_help:
-                   print_db_actions(self.common_automation)
+                   print_db_actions(self.common_automation, self.cfg['action_substitutions'])
                    
             return {'return':0, 'warning':'no action'}
 
         # Load info about all CM repositories (to enable search for automations and artifacts)
         if self.repos == None:
-            repos = Repos(path = self.home_path, cfg = self.cfg, 
+            repos = Repos(path = self.repos_path, cfg = self.cfg, 
                           path_to_internal_repo = self.path_to_cmind_repo)
 
             r = repos.load()
@@ -201,12 +234,12 @@ class CM(object):
         automation_lst = []
         use_any_action = False
 
-        artifact = i.get('artifact','')
+        artifact = i.get('artifact','').strip()
         artifacts = i.get('artifacts',[]) # Only if more than 1 artifact
                                           # First element is == artifact
 
         # Check if automation is "." - then attempt to detect repo, automation and artifact from the current directory
-        if automation == '' or automation == '.':
+        if automation =='' or automation == '.':
             r = self.access({'action':'detect',
                              'automation':'repo,55c3e27e8a140e48'})
             if r['return']>0: return r
@@ -218,16 +251,16 @@ class CM(object):
                 automation = ''
 
             # Check and make an artifact (only if artifacts are not specified)
-            if r.get('artifact_found_in_current_path', False) and len(artifacts)==0:
+            if r.get('artifact_found_in_current_path', False) and artifact == '':
                 artifact = r['cm_artifact']
 
-            if r.get('repo_found',False):
+            if r.get('registered', False):
                 cm_repo = r['cm_repo']
 
                 if ':' not in artifact:
                    artifact = cm_repo + ':' + artifact
    
-                for ia in len(artifacts):
+                for ia in range(0,len(artifacts)):
                     a = artifacts[ia]
                     if ':' not in a:
                         a = cm_repo + ':' + a
@@ -254,10 +287,13 @@ class CM(object):
                 auto_name = parsed_automation[0] if len(parsed_automation)>0 else ('','')
                 auto_repo = parsed_automation[1] if len(parsed_automation)>1 else None
 
-
                 # Search for automations in repos (local, internal, other) TBD: maybe should be local, other, internal?
-                r = self.common_automation.search({'parsed_automation':[('automation','bbeb15d8f0a944a4')],
-                                                   'parsed_artifact':parsed_automation})
+                ii={'parsed_automation':[('automation','bbeb15d8f0a944a4')],
+                    'parsed_artifact':parsed_automation}
+                # Ignore inheritance when called recursively
+                if i.get('ignore_inheritance',False):
+                   ii['ignore_inheritance']=True 
+                r = self.common_automation.search(ii)
                 if r['return']>0: return r
                 automation_lst = r['list']
 
@@ -270,6 +306,10 @@ class CM(object):
 
                     use_any_action = automation_meta.get('use_any_action',False)
                     
+                    # Update parsed_automation with UID and alias
+                    parsed_automation[0] = (automation_meta.get('alias',''),
+                                            automation_meta.get('uid',''))
+                    
                     # Find Python module for this automation
                     try:
                         found_automation = imp.find_module(automation_name, [automation_path])
@@ -356,12 +396,14 @@ class CM(object):
             
             print (self.cfg['info_cli'])
 
-            r = print_db_actions(self.common_automation)
+            print ('')
+            print ('Automation python module: {}'.format(automation_full_path))
+
+            r = print_db_actions(self.common_automation, self.cfg['action_substitutions'])
             if r['return']>0: return r
 
             db_actions = r['db_actions']
 
-
             actions = []
             for d in sorted(dir(initialized_automation)):
                 if d not in db_actions and type(getattr(initialized_automation, d))==types.MethodType and not d.startswith('_'):
@@ -411,9 +453,9 @@ class CM(object):
         if len(artifacts)>0:
             parsed_artifacts = []
 
-            for artifact in artifacts:
+            for extra_artifact in artifacts:
                 # Parse artifact
-                r = utils.parse_cm_object(artifact)
+                r = parse_cm_object_and_check_current_dir(self, extra_artifact)
                 if r['return'] >0 : return r
 
                 parsed_artifacts.append(r['cm_object'])
@@ -423,7 +465,7 @@ class CM(object):
         # Check artifact and artifacts
         if artifact != '':
             # Parse artifact
-            r = utils.parse_cm_object(artifact)
+            r = parse_cm_object_and_check_current_dir(self, artifact)
             if r['return'] >0 : return r
 
             i['parsed_artifact'] = r['cm_object']
@@ -431,17 +473,38 @@ class CM(object):
         # Call automation action
         action_addr=getattr(initialized_automation, action)
 
-#        import json
-#        print ('')
-#        print (json.dumps(i, indent=2))    
-#        print ('')        
-#        
         r = action_addr(i)
 
         return r
 
 ############################################################
-def print_db_actions(automation):
+def parse_cm_object_and_check_current_dir(cmind, artifact):
+    """
+    Internal function: parses CM object and check if there is '.' 
+    to detect CM repository in a current directory.
+
+    """
+
+    if artifact.startswith('.:'):
+
+        r = cmind.access({'action':'detect',
+                       'automation':'repo,55c3e27e8a140e48'})
+        if r['return']>0: return r
+
+        if r.get('registered', False):
+            cm_repo = r['cm_repo']
+
+            artifact = cm_repo + artifact[1:]
+    
+    return utils.parse_cm_object(artifact)
+
+############################################################
+def print_db_actions(automation, equivalent_actions):
+
+    """
+    Internal function: prints CM database actions.
+
+    """
 
     import types
 
@@ -453,10 +516,26 @@ def print_db_actions(automation):
 
     for d in sorted(dir(automation)):
         if type(getattr(automation, d))==types.MethodType and not d.startswith('_'):
-
+            
             db_actions.append(d)
 
-            print ('  * '+d)
+            s = d
+            se = ''
+
+            # Check equivalent actions
+            for k in sorted(equivalent_actions.keys()):
+                v = equivalent_actions[k]
+
+                if v==d:
+                    if se!='': se+=', '
+                    se+=k
+
+                    db_actions.append(k)
+
+            if se!='':
+                s+=' (' + se + ')'
+
+            print ('  * ' + s)
 
     return {'return':0, 'db_actions':db_actions}
 
@@ -464,7 +543,11 @@ def print_db_actions(automation):
 ############################################################
 def access(i):
     """
-    Initialize and access Collective Mind without customization
+    Automatically initialize CM and run automations 
+    without the need to initialize and customize CM class.
+    Useful for Python automation scripts.
+
+    See CM.access function for more details.
     """
 
     global cm
@@ -477,7 +560,11 @@ def access(i):
 ############################################################
 def error(i):
     """
-    Print error
+    Automatically initialize CM and print error if needed
+    without the need to initialize and customize CM class.
+    Useful for Python automation scripts.
+
+    See CM.error function for more details.
     """
 
     return cm.error(i)
@@ -485,15 +572,30 @@ def error(i):
 ############################################################
 def halt(i):
     """
-    Print error and halt
+    Automatically initialize CM, print error and halt if needed
+    without the need to initialize and customize CM class.
+    Useful for Python automation scripts.
+
+    See CM.halt function for more details.
+
     """
 
     return cm.halt(i)
 
 ############################################################
-def get_version(skip_check_status = False):
+def get_version():
     """
-    Get version and check status
+    Get CM version.
+
+    Args:
+
+    Returns: 
+        (CM return dict):
+
+        * return (int): return code == 0 if no error and >0 if error
+        * (error) (str): error string if return>0
+
+        * version (str): CM version
     """
 
     r = {'return':0}
