@@ -1,4 +1,6 @@
-﻿# Some functionality may be reused from the CK framework
+﻿# Auxilary functions for CM
+
+# Some functionality was reused from the CK framework for compatibility
 
 import os
 
@@ -9,6 +11,26 @@ ERROR_FILE_NOT_FOUND = 16
 ###########################################################################
 def load_yaml_and_json(file_name_without_ext, check_if_exists = False, encoding = 'utf8'):
 
+    """
+    Load YAML file if exists then JSON file if exists and merge with the first one
+
+    Args:    
+       (CM input dict):
+
+       file_name_without_ext (str): file name without extension (to check yaml and then json)
+       (check_if_exists) (bool): if True, fail if doesn't exist
+       (encoding) (str): file encoding ('utf8' by default)
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * meta (dict): merged meta of found file(s)
+
+    """
+
     meta = {}
 
     not_found = True
@@ -33,6 +55,24 @@ def load_yaml_and_json(file_name_without_ext, check_if_exists = False, encoding
 ###########################################################################
 def is_file_json_or_yaml(file_name):
 
+    """
+    Is file JSON or YAML?
+
+    Args:    
+       (CM input dict):
+
+       file_name (str): path to file without extension
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * is_file (bool): if True, there is a file with YAML and/or JSON extension
+
+    """
+
     for file_ext in ['.yaml', '.json']:
         file_path = file_name + file_ext
 
@@ -43,6 +83,25 @@ def is_file_json_or_yaml(file_name):
 
 ###########################################################################
 def load_json_or_yaml(file_name, check_if_exists = False, encoding = 'utf8'):
+    """
+    Attempt to load file as JSON or YAML.
+
+    Args:    
+       (CM input dict):
+
+       file_name (str): file name that has either JSON or YAML extension
+       (check_if_exists) (bool): if True, fail if doesn't exist
+       (encoding) (str): file encoding ('utf8' by default)
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * meta (dict): merged meta of found file(s)
+
+    """
 
     if file_name.endswith('.json'):
         return load_json(file_name, check_if_exists = check_if_exists, encoding = encoding)
@@ -53,6 +112,25 @@ def load_json_or_yaml(file_name, check_if_exists = False, encoding = 'utf8'):
 
 ###########################################################################
 def save_json_or_yaml(file_name, meta, sort_keys=False, encoding = 'utf8'):
+    """
+    Save meta to either JSON or YAML file.
+
+    Args:    
+       (CM input dict):
+
+       file_name (str): file name that has either JSON or YAML extension
+       meta (dict): meta to save
+       (sort_keys) (bool): if True, sort keys
+       (encoding) (str): file encoding ('utf8' by default)
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+    """
+
     if file_name.endswith('.json'):
         return save_json(file_name, meta, sort_keys, encoding = encoding)
     elif file_name.endswith('.yaml'):
@@ -62,6 +140,25 @@ def save_json_or_yaml(file_name, meta, sort_keys=False, encoding = 'utf8'):
 
 ###########################################################################
 def load_json(file_name, check_if_exists = False, encoding='utf8'):
+    """
+    Load JSON file.
+
+    Args:    
+       (CM input dict):
+
+       file_name (str): file name
+       (check_if_exists) (bool): if True, fail if doesn't exist
+       (encoding) (str): file encoding ('utf8' by default)
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * meta (dict): meta from the file
+
+    """
 
     if check_if_exists:
         import os
@@ -78,6 +175,25 @@ def load_json(file_name, check_if_exists = False, encoding='utf8'):
 
 ###########################################################################
 def save_json(file_name, meta={}, indent=2, sort_keys=True, encoding = 'utf8'):
+    """
+    Save meta to JSON file.
+
+    Args:    
+       (CM input dict):
+
+       file_name (str): file name 
+       meta (dict): meta to save
+       (indent) (int): 2 by default
+       (sort_keys) (bool): if True, sort keys
+       (encoding) (str): file encoding ('utf8' by default)
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+    """
 
     import json
     
@@ -88,6 +204,25 @@ def save_json(file_name, meta={}, indent=2, sort_keys=True, encoding = 'utf8'):
 
 ###########################################################################
 def load_yaml(file_name, check_if_exists = False, encoding = 'utf8'):
+    """
+    Load YAML file.
+
+    Args:    
+       (CM input dict):
+
+       file_name (str): file name
+       (check_if_exists) (bool): if True, fail if doesn't exist
+       (encoding) (str): file encoding ('utf8' by default)
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * meta (dict): meta from the file
+
+    """
 
     if check_if_exists:
         import os
@@ -105,6 +240,24 @@ def load_yaml(file_name, check_if_exists = False, encoding = 'utf8'):
 
 ###########################################################################
 def load_txt(file_name, encoding = 'utf8'):
+    """
+    Load text file.
+
+    Args:    
+       (CM input dict):
+
+       file_name (str): file name 
+       (encoding) (str): file encoding ('utf8' by default)
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * string (str): string from file
+
+    """
 
     with open(file_name, 'rt', encoding = encoding) as tf:
         s = tf.read()
@@ -112,8 +265,52 @@ def load_txt(file_name, encoding = 'utf8'):
     return {'return':0,
             'string': s}
 
+###########################################################################
+def load_bin(file_name):
+    """
+    Load binary file.
+
+    Args:    
+       (CM input dict):
+
+       file_name (str): file name 
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * bin (bytes): file content
+
+    """
+
+    with open(file_name, 'rb') as bf:
+        b = bf.read()
+
+    return {'return':0,
+            'bin': b}
+
 ###########################################################################
 def save_yaml(file_name, meta={}, sort_keys=True, encoding = 'utf8'):
+    """
+    Save meta to YAML file.
+
+    Args:    
+       (CM input dict):
+
+       file_name (str): file name 
+       meta (dict): meta to save
+       (sort_keys) (bool): if True, sort keys
+       (encoding) (str): file encoding ('utf8' by default)
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+    """
 
     import yaml
     
@@ -122,10 +319,46 @@ def save_yaml(file_name, meta={}, sort_keys=True, encoding = 'utf8'):
 
     return {'return':0}
 
+###########################################################################
+def save_txt(file_name, string = '', encoding = 'utf8'):
+    """
+    Save string to text file.
+
+    Args:    
+       (CM input dict):
+
+       file_name (str): file name 
+       string (str): string to save
+       (encoding) (str): file encoding ('utf8' by default)
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+    """
+
+    with open(file_name, 'wt', encoding = encoding) as tf:
+        tf.write(string)
+
+    return {'return':0}
+
 ###########################################################################
 def check_and_create_dir(path):
     """
-    Create directories if path doesn't exist
+    Create directories if path doesn't exist.
+    (from the CK framework).
+
+    Args:
+       path (str): path
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
     """
 
     if not os.path.isdir(path):
@@ -144,13 +377,14 @@ def find_file_in_dir_and_above(filename,
         path (str)
 
     Returns:
-        (dict) return (int): 0 - if found
-                             16 - if not found
-               (error) (str)
+       (CM return dict):
 
-               path (str): path where file is found
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * path (str): path where file is found
+       * path_to_file (str): path to file
 
-               path_to_file (str): path to file
     """
 
     if path == "":
@@ -178,35 +412,35 @@ def find_file_in_dir_and_above(filename,
 
 ##############################################################################
 def list_all_files(i):
-    """List all files recursively in a given directory
-       (from CK framework)
+    """
+    List all files recursively in a given directory.
+    (from the CK framework).
 
     Args:    
-              path (str): top level path
-              (file_name) (str): search for a specific file name
-              (pattern) (str): return only files with this pattern
-              (path_ext) (str): path extension (needed for recursion)
-              (limit) (str): limit number of files (if directories with a large number of files)
-              (number) (int): current number of files
-              (all) (str): if 'yes' do not ignore special directories (like .cm)
-              (ignore_names) (list): list of names to ignore
-              (ignore_symb_dirs) (str): if 'yes', ignore symbolically linked dirs 
-                                        (to avoid recursion such as in LLVM)
-              (add_path) (str) - if 'yes', add full path to the final list of files
+       (CM input dict):
+
+       path (str): top level path
+       (file_name) (str): search for a specific file name
+       (pattern) (str): return only files with this pattern
+       (path_ext) (str): path extension (needed for recursion)
+       (limit) (str): limit number of files (if directories with a large number of files)
+       (number) (int): current number of files
+       (all) (str): if 'yes' do not ignore special directories (like .cm)
+       (ignore_names) (list): list of names to ignore
+       (ignore_symb_dirs) (str): if 'yes', ignore symbolically linked dirs 
+                                 (to avoid recursion such as in LLVM)
+       (add_path) (str) - if 'yes', add full path to the final list of files
 
     Returns:
-              (dict): Unified CK dictionary:
+       (CM return dict):
 
-                return (int): return code =  0, if successful
-                                          >  0, if error
-                (error) (str): error text if return > 0
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
 
-                list (dict): dictionary of all files:
-                             {"file_with_full_path":{"size":.., "path":..}
+       * list (dict): dictionary of all files:
+                      {"file_with_full_path":{"size":.., "path":..}
 
-                sizes (dict): sizes of all files (the same order as above "list")
-
-                number (int): (internal) total number of files in a current directory (needed for recursion)
+       * number (int): (internal) total number of files in a current directory (needed for recursion)
 
     """
 
@@ -290,7 +524,19 @@ def list_all_files(i):
 ###########################################################################
 def gen_uid():
     """
-    Generate CM UID
+    Generate CM UID.
+
+    Args:    
+        None
+
+    Returns:
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * uid (str): CM UID
+
     """
 
     import uuid
@@ -301,13 +547,14 @@ def gen_uid():
 ###########################################################################
 def is_cm_uid(obj):
     """
-    Check if a string is a valid CM UID
+    Check if a string is a valid CM UID.
 
     Args:    
-              obj (str): CM alias or UID
+       obj (str): CM alias or UID
 
     Returns:
-              (bool): True if a string is a valid CK UID
+       (bool): True if valid CM UID 16 hex characters
+
     """
 
     import re
@@ -324,33 +571,40 @@ def is_cm_uid(obj):
 ###########################################################################
 def parse_cm_object(obj, max_length = 2):
     """
-    Parse CM object
+    Parse CM object in string and return tuple of CM objects.
+
+    Examples:
+        CM sub-object = UID | alias | alias,UID | UID,alias
+
+        repo CM sub-object | CM sub-object
+
+        cm os
+
+        cm 281d5c3e3f69d8e7
+        
+        cm os,281d5c3e3f69d8e7
+        
+        cm 281d5c3e3f69d8e7,os
+        
+        cm octoml@mlops,os
+        
+        cm octoml@mlops,dbfa91645e429380:os,281d5c3e3f69d8e7
+        
+        cm dbfa91645e429380:281d5c3e3f69d8e7
+
 
     Args:    
         obj (str): CM object
-
-                      CM sub-object = UID | alias | alias,UID | UID,alias
-                      repo CM sub-object | CM sub-object
-
-                      Examples:
-
-                      cm os
-                      cm 281d5c3e3f69d8e7
-                      cm os,281d5c3e3f69d8e7
-                      cm 281d5c3e3f69d8e7,os
-
-                      cm octoml@mlops,os
-                      cm octoml@mlops,dbfa91645e429380:os,281d5c3e3f69d8e7
-                      cm dbfa91645e429380:281d5c3e3f69d8e7
+        (max_length) (int): 
 
     Returns:
-        return (int): return code == 0 if no error 
-                                  >0 if error
+       (CM return dict):
 
-        (error) (str): error string if return>0
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
 
-        cm_object (list): first argument: CM alias | UID
-                          (second element: CM repo | UID)
+       * cm_object (list): first element: CM alias | UID
+                           (second element: CM repo | UID)
 
     """
 
@@ -390,33 +644,45 @@ def parse_cm_object(obj, max_length = 2):
 
 
 ###########################################################################
-def match_objects(uid, alias, uid2, alias2):
+def match_objects(uid, alias, uid2, alias2, more_strict = False):
     """
-    Check if 2 CM objects match
+    Check if 2 CM objects match. 
+    Used to search CM artifacts in CM repositories.
 
-    Args:    
+    Examples:
+            281d5c3e3f69d8e7,* == 281d5c3e3f69d8e7,*
 
-          alias can't have wildcards (real CM object)
-          alias2 can have wildcards (search)
-          
-          281d5c3e3f69d8e7,* == 281d5c3e3f69d8e7,*
-          281d5c3e3f69d8e7,os == ,os
-          ,os == 281d5c3e3f69d8e7,os
-          ,* != 281d5c3e3f69d8e7,*
+            281d5c3e3f69d8e7,os == ,os
+            
+            ,os == 281d5c3e3f69d8e7,os
+            
+            ,* != 281d5c3e3f69d8e7,*
 
-          os
-          281d5c3e3f69d8e7
-          os,281d5c3e3f69d8e7
-          281d5c3e3f69d8e7,os
+            os
+            
+            281d5c3e3f69d8e7
+            
+            os,281d5c3e3f69d8e7
+            
+            281d5c3e3f69d8e7,os
+
+    Args:
+       uid (str): artifact UID
+       alias (str): atifact alias that can't have wildcards [real CM object]
+
+       uid2 (str): atifact 2 UID
+       alias2 (str) artifact 2 alias that can have wildcards [search]
+
+      (more_strict) (bool): if True, then ,os != 281d5c3e3f69d8e7, [needed to check automation]
 
 
     Returns:
-        return (int): return code == 0 if no error 
-                                  >0 if error
+       (CM return dict):
 
-        (error) (str): error string if return>0
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
 
-        match (bool): True if 2 CM objects match (uid and/or alias)
+       * match (bool): True if 2 CM objects match (uid and/or alias)
 
     """
 
@@ -434,12 +700,13 @@ def match_objects(uid, alias, uid2, alias2):
     if uid!='' and uid2!='':
         if uid==uid2:
             match = True
+    elif more_strict and uid=='' and uid2!='' and alias2=='':
+        # Not match
+        pass
     else:
         # As soon as one UID is not there, we try to match by alias with wildcards
         # Both aliases must be present otherwise ambiguity - we report is as no match
-        object2_has_wildcards = False
-        if '*' in alias2 or '?' in alias2:
-            object2_has_wildcards = True
+        object2_has_wildcards = True if ('*' in alias2 or '?' in alias2) else False
 
         alias = alias.lower()
         alias2 = alias2.lower()
@@ -458,17 +725,14 @@ def match_objects(uid, alias, uid2, alias2):
 ###########################################################################
 def get_list_from_cli(i, key):
     """
-    Get list from a CLI 
+    Get list from CM command line.
 
     Args:    
+       i (dict): CM input dict
+       key (str): key to get
 
     Returns:
-        return (int): return code == 0 if no error 
-                                  >0 if error
-
-        (error) (str): error string if return>0
-
-        match (bool): True if 2 CM objects match (uid and/or alias)
+       tags (list): list of tags for a given key
 
     """
 
@@ -483,32 +747,33 @@ def get_list_from_cli(i, key):
 
 ##############################################################################
 def merge_dicts(i):
-    """Merge intelligently dict1 with dict2 key by key in contrast with dict1.update(dict2)
-       Target audience: end users
+    """
+    Merge intelligently dict1 with dict2 key by key in contrast with dict1.update(dict2)
 
-       It can merge sub-dictionaries and lists instead of substituting them
+    It can merge sub-dictionaries and lists instead of substituting them
 
     Args:    
-              dict1 (dict): merge this dict with dict2 (will be directly modified!)
-              dict2 (dict): dict to be merged
-              append_lists (str): if 'yes', append lists instead of creating the new ones
-              ignore_keys (list): ignore keys
+       (CM input dict): 
+
+       dict1 (dict): merge this dict with dict2 (will be directly modified!)
+       dict2 (dict): dict to be merged
+       (append_lists) (bool): if True, append lists instead of creating the new ones
+       (ignore_keys) (list): ignore keys
 
     Returns:
-              (dict): Unified CK dictionary:
+       (CM return dict):
 
-                return (int): return code =  0, if successful
-                                          >  0, if error
-                (error) (str): error text if return > 0
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
 
-                dict1 (dict): dict1 passed through the function
+       * dict1 (dict): dict1 passed through the function
 
     """
 
     a = i['dict1']
     b = i['dict2']
 
-    append_lists=i.get('append_lists','')
+    append_lists = i.get('append_lists', False)
 
     ignore_keys = i.get('ignore_keys',[])
 
@@ -524,7 +789,7 @@ def merge_dicts(i):
             else:
                 a[k] = b[k]
         elif type(v) is list:
-            if append_lists!='yes' or k not in a:
+            if not append_lists or k not in a:
                a[k] = []
             for y in v:
                 a[k].append(y)
@@ -535,23 +800,24 @@ def merge_dicts(i):
 
 ###########################################################################
 def process_meta_for_inheritance(i):
-    """Process meta for inheritance
+    """
+    Check CM meta description for inheritance and update it if needed ("_base":"automation::artifact").
 
     Args:
-              artifact (obj): CM artifact
-              meta (dict): original meta
-              cmind (obj): initialized CM to search for base artifacts
-              (base_recursion) (int): track recursion during inheritance
+       (CM input dict): 
+
+       automation (str): CM automation
+       meta (dict): original meta
+       cmind (obj): initialized CM to search for base artifacts
+       (base_recursion) (int): track recursion during inheritance
 
     Returns:
-              (dict): Unified CK dictionary:
+       (CM return dict):
 
-                return (int): return code =  0, if successful
-                                          >  0, if error
-                (error) (str): error text if return > 0
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
 
-                dict (dict):        CK updated meta with inheritance from base entries
-                (dict_orig) (dict): original CK meta if CK was updated with a base entry
+       * meta (dict): final meta description wihhout _base
 
     """
 
@@ -559,9 +825,11 @@ def process_meta_for_inheritance(i):
     current_meta = i.get('meta',{})
     cmind = i['cmind']
     
-    base_entry = current_meta.get('_base_artifact','').strip()
+    base_entry = current_meta.get('_base','').strip()
 
     if base_entry!='':
+        del (current_meta['_base'])
+
         base_recursion = int(i.get('base_recursion','0'))
 
         if base_recursion > 10:
@@ -597,31 +865,31 @@ def process_meta_for_inheritance(i):
         base_meta = base_artifact.meta
 
         r = merge_dicts({'dict1': base_meta, 
-                         'dict2': current_meta})
+                         'dict2': current_meta,
+                         'append_lists': True})
         if r['return']>0: return r
 
-        current_meta = base_meta
+        current_meta = r['dict1']
 
     return {'return':0, 'meta':current_meta}
 
 
 ###########################################################################
 def find_api(file_name, func):
-    """Find func API in filename
+    """
+    Find automation action API in a Python module
 
     Args:
-              file_name (str): Python module
-              func (str): func name
+       file_name (str): Python module
+       func (str): automation action name
 
     Returns:
-              (dict): Unified CK dictionary:
+       (CM return dict):
 
-                return (int): return code =  0, if successful
-                                          >  0, if error
-                (error) (str): error text if return > 0
-
-                api (str): API string
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
 
+       * api (str): API
     """
 
     # Load file
@@ -654,24 +922,24 @@ def find_api(file_name, func):
 
 ###########################################################################
 def find_file_in_current_directory_or_above(file_names, path_to_start = None, reverse = False):
-    """Find file in the current directory or above
+    """
+    Find file(s) in the current directory or above.
 
     Args:
-              file_names (list): files to find
-              path_to_start (str): path to start
-                                   use current directory if None
+       file_names (list): files to find
+       (path_to_start) (str): path to start; use current directory if None
+       (reverse) (bool): if True search recursively in current directory and below.
 
     Returns:
-              (dict): Unified CK dictionary:
+       (CM return dict):
 
-                return (int): return code =  0, if successful
-                                          >  0, if error
-                (error) (str): error text if return > 0
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
 
-                found (bool): True if found
+       * found (bool): True if found
 
-                path_to_file (str): full path to found file
-                path (str): path where found file is
+       * path_to_file (str): full path to found file
+       * path (str): path where found file is
 
     """
 
@@ -735,25 +1003,15 @@ def find_file_in_current_directory_or_above(file_names, path_to_start = None, re
 
 ###########################################################################
 def assemble_cm_object(alias,uid):
-    """Assemble CM object
+    """
+    Assemble CM object string from alias and uid strings
 
-    Args:
-              file_names (list): files to find
-              path_to_start (str): path to start
-                                   use current directory if None
-
-    Returns:
-              (str): CM object
-
-                return (int): return code =  0, if successful
-                                          >  0, if error
-                (error) (str): error text if return > 0
-
-                found (bool): True if found
-
-                path_to_file (str): full path to found file
-                path (str): path where found file is
+    Args:    
+       alias (str): CM artifact alias
+       uid (str): CM artifact UID
 
+    Returns: 
+       (str) CM object (alias,uid)
     """
 
     if alias == None: alias = ''
@@ -771,3 +1029,144 @@ def assemble_cm_object(alias,uid):
            cm_obj = alias
 
     return cm_obj
+
+###########################################################################
+def assemble_cm_object1(cm_dict):
+    """
+    Assemble CM object string from dictionary with 'alias' and 'uid' keys
+
+    Args:    
+       cm_dict (dict): dictionary with CM artifact keys ('alias', 'uid')
+
+    Returns: 
+       (str) CM object (alias,uid)
+
+    """
+
+    return assemble_cm_object(cm_dict['alias'],cm_dict['uid'])
+
+###########################################################################
+def assemble_cm_object2(cm_obj):
+    """
+    Assemble CM object string from tuple
+
+    Args:    
+       cm_obj (tuple): CM object tuple (alias, uid)
+
+    Returns: 
+       (str) CM object (alias,uid)
+
+    """
+
+    return assemble_cm_object(cm_obj[0], cm_obj[1])
+
+###########################################################################
+def dump_safe_json(i):
+    """
+    Dump safe JSON
+
+    Args:    
+       (CM input dict): input to dump to console
+
+    Returns: 
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * meta (dict): only "safe" values left (that can be serialized)
+
+    """
+
+    import json
+    
+    meta = {}
+
+    for k in i:
+        v = i[k]
+        
+        try:
+           s = json.dumps(v)
+        except Exception as e:
+           pass
+        else:
+           meta[k] = v
+
+    print (json.dumps(meta, indent=2, sort_keys=True, ensure_ascii=False))
+
+    return {'return':0, 'meta': meta}
+
+###########################################################################
+def convert_tags_to_list(i):
+    """
+    Convert string tags to list
+
+    Args:    
+       (CM input dict): 
+
+       (tags) (str) - string of tags separated by comma
+
+    Returns: 
+       tags_list (list): list of tags
+    
+    """
+
+    tags = i.get('tags')
+    if tags == None: tags=''
+
+    tags = tags.strip()
+
+    tags_list = []
+
+    if tags!='': 
+       tags_list_tmp = tags.split(',')
+
+       for tag in tags_list_tmp:
+           tag = tag.strip()
+
+           if tag not in tags_list:
+               tags_list.append(tag)
+
+    return tags_list
+
+##############################################################################
+def get_current_date_time(i):
+    """
+    Get current date and time.
+
+    Args:    
+       (CM input dict): empty dict
+
+    Returns: 
+       (CM return dict):
+
+       * return (int): return code == 0 if no error and >0 if error
+       * (error) (str): error string if return>0
+
+       * array (dict); dict with date and time
+
+           - date_year (str)
+           - date_month (str)
+           - date_day (str)
+           - time_hour (str)
+           - time_minute (str)
+           - time_second (str)
+
+       * iso_datetime (str): date and time in ISO format
+    """
+
+    import datetime
+
+    a = {}
+
+    now1 = datetime.datetime.now()
+    now = now1.timetuple()
+
+    a['date_year'] = now[0]
+    a['date_month'] = now[1]
+    a['date_day'] = now[2]
+    a['time_hour'] = now[3]
+    a['time_minute'] = now[4]
+    a['time_second'] = now[5]
+
+    return {'return': 0, 'array': a, 'iso_datetime': now1.isoformat()}
